//---------------------------------------------------------------------
//
// DispFntDlg.cpp : implementation file
//
// Contains the code to process the Show... windows, specifically
// the Show Character Set and Show Alphanumerics.
//
//
// Part of the FontViewer Utility, which first appeared
// in the May 26, 1998 issue of PC Magazine, U.S. Edition.
// Copyright (C) 1998 by Ziff-Davis, Inc. 
// Programmed by Jake Richter.
//
// The programs presented in PC Magazine can be copied but is copyright.
// You may make copies for others if no charge is involved, but copies
// or using any portion for any commerical purpose is strictly prohibited.
//
// For support, go to http://www.pcmag.com/discuss.htm and select 
// the Utilities area.
//
//---------------------------------------------------------------------

#include "stdafx.h"
#include "FViewer.h"
#include "FViewerDoc.h"
#include "FViewerView.h"
#include "DispFntDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Local info

#define  WHTBRDR     20    // Amount of border space, in pixels for the AlphaNum display
#define  NUMSTRS      3    // Number of strings for the AlphaNum display

static   CString  dispStrings[NUMSTRS] = {"ABCDEFGHIJKLMNOPQRSTUVWXYZ",  // AlphaNum strings
                              "abcdefghijklmnopqrstuvwxyz",
                              "0123456789!@#$%^&*()+-={}[],."};

/////////////////////////////////////////////////////////////////////////////
// CDispFntDlg dialog


CDispFntDlg::CDispFntDlg(CWnd* pParent /*=NULL*/)
   : CDialog(CDispFntDlg::IDD, pParent)
{
   m_typeOfShow = TRUE;    // Default is grid
   //{{AFX_DATA_INIT(CDispFntDlg)
      // NOTE: the ClassWizard will add member initialization here
   //}}AFX_DATA_INIT
   return;
}

void CDispFntDlg::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
   //{{AFX_DATA_MAP(CDispFntDlg)
      // NOTE: the ClassWizard will add DDX and DDV calls here
   //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDispFntDlg, CDialog)
   //{{AFX_MSG_MAP(CDispFntDlg)
   ON_WM_PAINT()
    ON_WM_LBUTTONDOWN()
    ON_WM_LBUTTONUP()
   ON_WM_MOUSEMOVE()
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------
//
// CDispFntDlg message handlers
//
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//
// DoModal()
//
// An override called to create and display the dialog box. Here we
// set the flags/values that need to be set each time the dialog box
// is brought up.
//
//---------------------------------------------------------------------
int CDispFntDlg::DoModal() 
{
   m_fontSize = FONTSIZE;        // Starting font size
   m_mouseClickDown = FALSE;     // Initialize mouse button state
   
   return CDialog::DoModal();
}

//---------------------------------------------------------------------
//
// OnLButtonUp()
//
// We detect the second part of a Left Button click here. Once the Left
// Button has come up, we can exit, as a left click anywhere in our
// client window will result in a termination of the window.
//
//---------------------------------------------------------------------
void CDispFntDlg::OnLButtonUp(UINT nFlags, CPoint point)
{
   CDialog::OnLButtonUp(nFlags, point);
   if (m_mouseClickDown)      // Only a real left click if we got down event
      {
      m_mouseClickDown = FALSE;  // Reset the state
      EndDialog(IDOK);           // Outta here
      }
   return;
}

//---------------------------------------------------------------------
//
// OnLButtonDown()
//
// We get called here when the user presses down on the left mouse
// button, at which point we set a flag to indicate that this event
// has happened. This is necessary because otherwise we might still
// be in the middle of a mouse button down event from when the dialog
// window was brought up, resulting in a window that would be shut
// as soon as the user released the button button (the up event).
//
//---------------------------------------------------------------------
void CDispFntDlg::OnLButtonDown(UINT nFlags, CPoint point)
{
   CDialog::OnLButtonDown(nFlags, point);
   m_mouseClickDown = TRUE;
   return;
}

//---------------------------------------------------------------------
//
// OnPaint()
//
// In this dialog box, the OnPaint function acts as a redirector to
// one of the two types of font displays we have. The ShowXXXX functions
// do the real repainting, and resizing of the dialog.
//
//---------------------------------------------------------------------

void CDispFntDlg::OnPaint() 
{
   if (m_typeOfShow)
      ShowCharSet();       // Pull up grid display
   else
      ShowAlphaNums();

   return;
}

//---------------------------------------------------------------------
//
// ShowCharSet()
//
// Shows the entire font in a grid arrangement, and uses dynamic resizing
// to create a "best fit" window, as well as move various dialog box
// components around so that they appear where we want them to.
//
// It's almost guaranteed that this routine will be called twice (or even
// more times). The first time it will calculate the desired window size
// and tell Windows to resize the window. The second time it may do this
// again, but with a slightly smaller font, in case the window is too large
// to be completely visible on the screen. The final time it is called
// will be when the size is fine and we can display stuff in the window.
//
//---------------------------------------------------------------------
void CDispFntDlg::ShowCharSet() 
{
   int      maxXExt = 0;      // The maximum X extent for the grid box
   int      maxYExt = 0;      // The maximum Y extent for the grid box
   int      butHeight;        // The height & width of the OK button
   int      butWidth;
   int      txtHeight;        // The height & width of the dialog's text strings
   int      txtWidth;
   int      maxCtrlHeight;    // The maximum dimensions of the data display
   int      maxCtrlWidth;
   TEXTMETRIC myTextMet;      // Text Metrics (used for grid spacing)
   CRect rect;                // Client rectangle
   CRect winRect;             // Dialog rectangle, in screen coords.
   CRect butRect;             // Button coordinates
   CRect txtRect;             // Coordinates of one of four text controls
   CWnd* pTxtWnd = GetDlgItem(IDC_DF_ASCII); // Window pointer for ASCII label string
   CWnd* pButWnd = GetDlgItem(IDOK);   // Window pointer for OK Button
   CPaintDC dc(this);         // device context for painting
   CPaintDC butDC(pButWnd);   // device context for buttons
   CPaintDC txtDC(pTxtWnd);   // device context for text

   pButWnd->GetClientRect(butRect);    // Get the size of the button rect
   butHeight = (butRect.bottom - butRect.top);
   butWidth = (butRect.right - butRect.left);

   pTxtWnd->GetClientRect(txtRect);    // Get the size of the text string rects
   txtHeight = (txtRect.bottom - txtRect.top);
   txtWidth = (txtRect.right - txtRect.left);

                                       // Get the max control height
   maxCtrlHeight = butHeight > (txtHeight * 2 + 2) ? butHeight:(txtHeight * 2 + 2);
   maxCtrlWidth = butWidth + (txtWidth * 2 + 2);   // "+2s" are for spacing

   maxCtrlHeight += 8;        // Add a small white space buffer to each
   maxCtrlWidth += 8;         // Height is used to make dialog larger

   GetClientRect(rect);       // Get the size of the client rect
   SetWindowText(m_fullName); // Put Font name in title area

   dc.SetTextAlign(TA_TOP | TA_CENTER);  // Set text display parameters
   dc.SetTextColor(RGB(0,0,0));
   dc.SetBkMode(TRANSPARENT);

   CFont* pMyFont = new CFont;    // Create a new CFont for use below

                              // Select the font into the device context
   m_fontNode->m_logFont.lfHeight = m_fontSize - 20;  // Set point size (grid font 2pts smaller)
   pMyFont->CreatePointFontIndirect(&(m_fontNode->m_logFont), NULL);
   CFont *pOldFont = dc.SelectObject(pMyFont);
   dc.GetTextMetrics(&myTextMet);         // Get font info

   //
   // If the dialog isn't the right size (very likely since it starts out tiny),
   // make it larger. 
   // We do this by figuring out what the dimensions of our grid layout should
   // be via the text metrics function. Each grid cell will have a one pixel border
   // of white space around each character cell, plus one pixel of black, for a
   // total of 3. The grid is arrayed as 16x16 for a total of 256 cells.
   //
   maxXExt = 16 * (myTextMet.tmMaxCharWidth + 3);
   maxYExt = 16 * (myTextMet.tmHeight + 3) + maxCtrlHeight;

   int      xDelta = maxXExt - (rect.right - rect.left) - 1; // Figure out our growth.
   int      yDelta = maxYExt - (rect.bottom - rect.top) - 1;
   int      screenXSize, screenYSize;

   if (xDelta || yDelta)      // Do we need to resize? Yes if our deltas are non-0
      {
      dc.SelectObject( pOldFont );     // Switch back to old font
      pMyFont->DeleteObject();         // Free up GDI resource

      GetWindowRect(winRect);          // Get current screen position of dialog
      winRect.top -= yDelta / 2;       // Expand/shrink equally in all directions
      winRect.bottom += yDelta - yDelta / 2;
      winRect.left -= xDelta / 2;
      winRect.right += xDelta - xDelta / 2;
      
      //
      // Next, lets make sure the dialog is not off the screen, or
	  // overlapping any toolbars (unless its a bitmap font of a certain size,
	  // in which case we have very little flexibility.
      //
      RECT  workArea;	
      SystemParametersInfo(SPI_GETWORKAREA, 0, &workArea, 0);
      screenXSize = workArea.right - workArea.left;
      screenYSize = workArea.bottom - workArea.top;

      if (winRect.left < workArea.left)
         {
         winRect.right += workArea.left - winRect.left;
         winRect.left = workArea.left;
         }
      else
         if (winRect.right > workArea.right)
            {
            winRect.left -= winRect.right - workArea.right;
            winRect.right = workArea.right;
            }

      if (winRect.top < workArea.top)
         {
         winRect.bottom += workArea.top - winRect.top;
         winRect.top = workArea.top; 
         }
      else
         if (winRect.bottom > workArea.bottom)
            {
            winRect.top -= winRect.bottom - workArea.bottom;
            winRect.bottom = workArea.bottom;
            }

      //
      // Now, let's check and make sure the desired dialog isn't larger 
      // than the display!
      //
      if ((winRect.right - winRect.left > screenXSize) ||
         (winRect.bottom - winRect.top > screenYSize))
         {
		 if (myTextMet.tmPitchAndFamily & TMPF_VECTOR)  // If it's a vector font, try to resize.
			{
			// If the desired dialog is larger, we need to step down the font size by a
			// point (10 units) and try again until it fits.
			m_fontSize -= 10;
			Invalidate();     // Force a call back here to try it all again.
			delete pMyFont;   // Delete our created object
			return;
			}
         }

      //
      // Otherwise, resize the dialog box
      //
      MoveWindow(winRect.left, winRect.top,
               winRect.right - winRect. left,
               winRect.bottom - winRect.top,
               TRUE);               // Tell Windows to resize/move the dialog
      Invalidate();        // Force a call back here to try it all again.
      delete pMyFont;   // Delete our created object
      return;
      }

   //
   // This is where we actually start drawing stuff.
   //
   int      xPos = -1;        // The positioning value for the X for centering
   int      yPos = -1;        // Start of the Y drawing position
   int      xStep, yStep;     // The X and Y dimensions of a cell.
   int      i, j;             // Loop variables
   int      charVal = 0;      // The current character.
   CString charStr = " ";     // Set to a space.
   CPen* pPen = new CPen(PS_SOLID, 1, RGB(0,0,0));
   CPen* pOldPen = dc.SelectObject(pPen);

   //
   // The following code positions the OK button below the displayed grid, and
   // also positions and turns on the four static text strings.
   //
   pButWnd->MoveWindow(rect.right - (butWidth + 4), rect.bottom - (butHeight + 4),
                butWidth, butHeight, TRUE);

   pButWnd->ShowWindow(SW_SHOWNORMAL); // Make sure it's visible
   pButWnd->Invalidate();              // Force the button to be redrawn

   //
   // Position the strings next, then show them and redraw them
   //
   pTxtWnd = GetDlgItem(IDC_DF_ASCII);
   pTxtWnd->MoveWindow(rect.left + 4, rect.bottom - (txtHeight * 2 + 6),
                txtWidth, txtHeight, TRUE);
   pTxtWnd->ShowWindow(SW_SHOWNORMAL);
   pTxtWnd->Invalidate();

   pTxtWnd = GetDlgItem(IDC_DF_ASCIIVALUE);
   pTxtWnd->MoveWindow(rect.left + 4 + txtWidth + 2, rect.bottom - (txtHeight * 2 + 6),
                txtWidth, txtHeight, TRUE);
   pTxtWnd->ShowWindow(SW_SHOWNORMAL);
   pTxtWnd->Invalidate();

   pTxtWnd = GetDlgItem(IDC_DF_KEYSTROKE);
   pTxtWnd->MoveWindow(rect.left + 4, rect.bottom - (txtHeight + 4),
                txtWidth, txtHeight, TRUE);
   pTxtWnd->ShowWindow(SW_SHOWNORMAL);
   pTxtWnd->Invalidate();
   
   pTxtWnd = GetDlgItem(IDC_DF_KEYVALUE);
   pTxtWnd->MoveWindow(rect.left + 4 + txtWidth + 2, rect.bottom - (txtHeight + 4),
                txtWidth, txtHeight, TRUE);
   pTxtWnd->ShowWindow(SW_SHOWNORMAL);
   pTxtWnd->Invalidate();

   dc.FillSolidRect(rect.left, rect.top,  // Clear grid area to white
                  rect.right - rect.left, rect.bottom - rect.top - maxCtrlHeight,
                  RGB(255, 255, 255));

   xStep = myTextMet.tmMaxCharWidth + 3;  // Calculate grid cell size
   yStep = myTextMet.tmHeight + 3;

   for (i = 1; i <= 16; i++)  // Draw vertical & horizontal lines in grid.
      {
      dc.MoveTo(xPos + i * xStep, rect.top);
      dc.LineTo(xPos + i * xStep, rect.bottom - maxCtrlHeight);
      dc.MoveTo(rect.left, yPos + i * yStep);
      dc.LineTo(rect.right, yPos + i * yStep);
      }

   //
   // Fill in the grid.
   //
   for (i = 0; i < 16; i++)      // Rows
      {
      for (j = 0; j < 16; j++)   // Columns
         {
         if (charVal < myTextMet.tmFirstChar || charVal > myTextMet.tmLastChar)
            {                    // Draw a gray box if not a valid character.
            dc.FillSolidRect(j * xStep, i * yStep,
                  xStep - 1, yStep - 1,
                  RGB(192, 192, 192));
            }
         else
            {                    // Or, draw the character...
            charStr.SetAt(0, (char)charVal);
            dc.TextOut(j * xStep + 1 + ((xStep - 1) / 2), i * yStep + 1, charStr);
            }
         charVal++;              // Bump the character counter after each cell
         }
      }

   //
   // We're done, restore everything.
   //
   dc.SelectObject(pOldPen);
   dc.SelectObject(pOldFont);             // Switch back to old font
   delete pPen;
   delete pMyFont;                        // Delete our created objects

   m_xStep = myTextMet.tmMaxCharWidth + 3;   // Save for use in OnMouseMove()
   m_yStep = myTextMet.tmHeight + 3;
   m_displayed = TRUE;                    // Indicates m_xStep/m_yStep are valid

   m_mouseClickDown = FALSE;              // Reset click state after a paint.

   return;
}

//---------------------------------------------------------------------
//
// ShowAlphaNums
//
// This version of the OnPaint function produces a dialog window 
// several sample strings showing the alphabet, numbers, and some
// punctuation. It also moves the OK button to appear where we want it to.
//
// It's almost guaranteed that this routine will be called twice (or even
// more times). The first time it will calculate the desired window size
// and tell Windows to resize the window. The second time it may do this
// again, but with a slightly smaller font, in case the window is too large
// to be completely visible on the screen. The final time it is called
// will be when the size is fine and we can display stuff in the window.
//
//---------------------------------------------------------------------
void CDispFntDlg::ShowAlphaNums() 
{
   int      yPos;             // Start of the Y drawing position
   int      xPos;             // The positioning value for the X for centering
   int      maxXExt = 0;      // The maximum X extent for all displayed strings
   int      maxYExt = 0;      // The maximum Y extent for all displayed strings
   TEXTMETRIC myTextMet;      // Text Metrics (used for line spacing)
   CRect rect;                // Client rectangle
   CRect butRect;             // OK button rectangle
   int      butHeight;        // Button dimensions
   int      butWidth;
   CSize textExt;             // Size of the text string in pixels
   int      i;                // Loop variable
   CWnd* pButWnd = GetDlgItem(IDOK);   // Window pointer for OK Button
   CPaintDC butDC(pButWnd);   // device context for button
   CPaintDC dc(this);         // device context for painting


   pButWnd->GetClientRect(butRect);    // Get the size of the button
   butHeight = (butRect.bottom - butRect.top);
   butWidth = (butRect.right - butRect.left);

   GetClientRect(rect);       // Get the size of the client rect

   SetWindowText(m_fullName); // Put Font name in title area

   dc.SetTextAlign(TA_TOP | TA_CENTER);  // Set text display parameters
   dc.SetTextColor(RGB(0,0,0));
   dc.SetBkMode(TRANSPARENT);

   CFont* pMyFont = new CFont;// Create a new CFont for use below

                              // Select the font into the device context
   m_fontNode->m_logFont.lfHeight = m_fontSize;       // Set point size
   pMyFont->CreatePointFontIndirect(&(m_fontNode->m_logFont), NULL);
   CFont *pOldFont = dc.SelectObject(pMyFont);
   dc.GetTextMetrics(&myTextMet);         // Get font info

   //
   // If the dialog isn't the right size, make it larger.
   // We do this by getting the largest X and Y extent of each string to 
   // display, and then comparing against the client area, including a
   // white space buffer around everything.
   //
   for (i = 0; i < NUMSTRS; i++)
      {
      textExt = dc.GetTextExtent(dispStrings[i]);
      if (textExt.cx > maxXExt)
         maxXExt = textExt.cx;
      if (textExt.cy > maxYExt)
         maxYExt = textExt.cy;
      }

   if (((maxXExt + (WHTBRDR * 2) != (rect.right - rect.left)) ||
      (maxYExt * NUMSTRS + (WHTBRDR * 2) + butHeight + 8) != (rect.bottom - rect.top)))
      {     // Need to resize the dialog box to fit the text properly!
      CRect winRect;
      int      xDelta = (maxXExt + (WHTBRDR * 2)) - (rect.right - rect.left);
      int      yDelta = (maxYExt * NUMSTRS + (WHTBRDR * 2) + butHeight + 8)
                     - (rect.bottom - rect.top);
      int      screenXSize, screenYSize;

      dc.SelectObject( pOldFont );     // Switch back to old font
      pMyFont->DeleteObject();         // Free up GDI resource

      GetWindowRect(winRect);          // Get current screen position of dialog
      winRect.top -= yDelta / 2;       // Expand/shrink equally in all directions
      winRect.bottom += yDelta - yDelta / 2;
      winRect.left -= xDelta / 2;
      winRect.right += xDelta - xDelta / 2;
      
      //
      // Next, lets make sure the dialog is not off the left 
      // or top of the screen
      //
      if (winRect.left < 0)
         {
         winRect.right += -winRect.left;
         winRect.left = 0;
         }

      if (winRect.top < 0)
         {
         winRect.bottom += -winRect.top;
         winRect.top = 0;
         }

      //
      // Now, let's check and make sure the desired dialog isn't 
      // larger than the display!
      //
      RECT  workArea;	
      SystemParametersInfo(SPI_GETWORKAREA, 0, &workArea, 0);
      screenXSize = workArea.right - workArea.left;
      screenYSize = workArea.bottom - workArea.top;

      if (winRect.left < workArea.left)
         {
         winRect.right += workArea.left - winRect.left;
         winRect.left = workArea.left;
         }
      else
         if (winRect.right > workArea.right)
            {
            winRect.left -= winRect.right - workArea.right;
            winRect.right = workArea.right;
            }

      if (winRect.top < workArea.top)
         {
         winRect.bottom += workArea.top - winRect.top;
         winRect.top = workArea.top; 
         }
      else
         if (winRect.bottom > workArea.bottom)
            {
            winRect.top -= winRect.bottom - workArea.bottom;
            winRect.bottom = workArea.bottom;
            }

      //
      // Now, let's check and make sure the desired dialog isn't larger 
      // than the display!
      //
      if ((winRect.right - winRect.left > screenXSize) ||
         (winRect.bottom - winRect.top > screenYSize))
         {
		 if (myTextMet.tmPitchAndFamily & TMPF_VECTOR)  // If it's a vector font, try to resize.
			{
			// If the desired dialog is larger, we need to step down the font size by a
			// point (10 units) and try again until it fits.
			m_fontSize -= 10;
			Invalidate();     // Force a call back here to try it all again.
			delete pMyFont;   // Delete our created object
			return;
			}
         }

      // Otherwise, resize the dialog box
      MoveWindow(winRect.left, winRect.top,
               winRect.right - winRect. left,
               winRect.bottom - winRect.top,
               TRUE);   // Tell Windows to resize/move the dialog
      Invalidate();     // Force a call back here to try it all again.
      delete pMyFont;   // Delete our created object
      return;           // Need to exit, will get called to repaint
      }

   //
   // Here's where we actually draw stuff.
   //

   //
   // The following code positions the OK button below the displayed text.
   //
   xPos = rect.left + (rect.right - rect.left) / 2;  // Calculate X center of dialog
   yPos = rect.top + WHTBRDR;

   pButWnd->MoveWindow(xPos - (butWidth / 2),
                       rect.bottom - (butHeight + 4),
                       butWidth, butHeight, TRUE);
   pButWnd->ShowWindow(SW_SHOWNORMAL);
   pButWnd->Invalidate();                 // Force the button to be redrawn

   dc.FillSolidRect(rect.left, rect.top,  // Clear client rect to white
                  rect.right - rect.left, rect.bottom - rect.top,
                  RGB(255, 255, 255));

   for (i = 0; i < NUMSTRS; i++)
      {
      dc.TextOut(xPos, yPos, dispStrings[i]);
      yPos += maxYExt;                    // Space evenly horizontally
      }

   dc.SelectObject( pOldFont );           // Switch back to old font
   delete pMyFont;                        // Delete our created object

   m_mouseClickDown = FALSE;              // Reset click state after a paint.

   return;
}

//---------------------------------------------------------------------
//
// OnInitDialog()
//
// We use this call, made right before the dialog box is displayed,
// to disable/make invisible the five different dialog box controls
// in our display font dialog box. The individual OnDraw() called functions
// will enable (and reposition) those controls deemed necessary for the type
// of display operation.
//
//---------------------------------------------------------------------
BOOL CDispFntDlg::OnInitDialog() 
{
   CWnd* pTmpWnd;

   pTmpWnd = GetDlgItem(IDOK);
   pTmpWnd->ShowWindow(SW_HIDE);

   pTmpWnd = GetDlgItem(IDC_DF_ASCII);
   pTmpWnd->ShowWindow(SW_HIDE);

   pTmpWnd = GetDlgItem(IDC_DF_ASCIIVALUE);
   pTmpWnd->ShowWindow(SW_HIDE);

   pTmpWnd = GetDlgItem(IDC_DF_KEYSTROKE);
   pTmpWnd->ShowWindow(SW_HIDE);
   
   pTmpWnd = GetDlgItem(IDC_DF_KEYVALUE);
   pTmpWnd->ShowWindow(SW_HIDE);

   m_displayed = FALSE;

   CDialog::OnInitDialog();
   
   return TRUE;
}

//---------------------------------------------------------------------
//
// OnMouseMove()
//
// This function is called whenever the mouse is moved. We use this
// function to update the data area in the Show Character Set version
// of this dialog. The data area contents reflect the numerica ASCII
// value of the character the mouse is currently over, as well as the
// keystroke necessary to get the currently pointed to character.
//
//---------------------------------------------------------------------
void CDispFntDlg::OnMouseMove(UINT nFlags, CPoint point) 
{
   int      asciiValue;          // Stores the ASCII value of the current cell
   char  stringASCII[2] = " ";   // Temporary string to display key stroke
   char  tmp[10];                // Buffer for _atoi()
   CString  stringAlt = "Alt 0"; // Prefix string for keystroke

   //
   // We have to make sure that the mouse is actually over our
   // window, that our window is active, and that the Show Character Set
   // display is active. If any of these conditions doesn't hold true, we
   // punt.
   //
   if (GetActiveWindow() != this || !m_displayed)
      {
      CDialog::OnMouseMove(nFlags, point);
      return;
      }

   //
   // If we get here, we can be assured that m_xStep & m_yStep are valid.
   //
   asciiValue = point.x / m_xStep + (point.y / m_yStep * 16);  // Get ASCII
   if (asciiValue <= 255)     // Makes sure that we're in the grid.
      {
      _itoa(asciiValue, tmp, 10);   // Make the numeric string 
      SetDlgItemText(IDC_DF_ASCIIVALUE, tmp);   // Set the ASCII Value string
      if (asciiValue < 32)          // If it's a control character...
         {
         stringAlt += "0";          // Add an extra leading 0.
         stringAlt += tmp;          // and the numeric value.
         SetDlgItemText(IDC_DF_KEYVALUE, stringAlt);  // Display it
         }
      else
         {
         if (asciiValue > 126)      // Otherwise, if not a typable character
            {
            stringAlt += tmp;       // Display standard "Alt" string
            SetDlgItemText(IDC_DF_KEYVALUE, stringAlt);
            }
         else
            {                       // We have a typable character. Display it.
            stringASCII[0] = asciiValue;  // Use C style string handling
            SetDlgItemText(IDC_DF_KEYVALUE, stringASCII);
            }
         }
      }
   else
      {                             // If not in valid cell, display blanks
      SetDlgItemText(IDC_DF_ASCIIVALUE, " ");
      SetDlgItemText(IDC_DF_KEYVALUE, " ");
      }

   CDialog::OnMouseMove(nFlags, point);   // Move on for other processing
   return;
}
