//---------------------------------------------------------------------
//
// FViewerDoc.cpp : implementation of the CFViewerDoc class
//
// This file contains the Document handling code for FontViewer.
// The only real thing done here is the inclusion of a number of
// functions necessary to operating the CFontNode and CFontList 
// classes, which are used to store our font list (and a C++ alternative
// to using structures for such work). All the CFViewerDoc code was
// generated dynamically by AppWizard.
//
// Part of the FontViewer Utility, which first appeared
// in the May 26, 1998 issue of PC Magazine, U.S. Edition.
// Copyright (C) 1998 by Ziff-Davis, Inc. 
// Programmed by Jake Richter.
//
// The programs presented in PC Magazine can be copied but is copyright.
// You may make copies for others if no charge is involved, but copies
// or using any portion for any commerical purpose is strictly prohibited.
//
// For support, go to http://www.pcmag.com/discuss.htm and select 
// the Utilities area.
//
//---------------------------------------------------------------------

#include "stdafx.h"
#include "FViewer.h"
#include "MainFrm.h"
#include "FViewerDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFViewerDoc

IMPLEMENT_DYNCREATE(CFViewerDoc, CDocument)

BEGIN_MESSAGE_MAP(CFViewerDoc, CDocument)
   //{{AFX_MSG_MAP(CFViewerDoc)
      // NOTE - the ClassWizard will add and remove mapping macros here.
      //    DO NOT EDIT what you see in these blocks of generated code!
   //}}AFX_MSG_MAP
END_MESSAGE_MAP()
//--------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// CFViewerDoc construction/destruction
//
// During the Constructor, we enumerate all the fonts in the system.
//
////////////////////////////////////////////////////////////////////////////
CFViewerDoc::CFViewerDoc()
{
   return;
}

CFViewerDoc::~CFViewerDoc()
{
}

BOOL CFViewerDoc::OnNewDocument()
{
   if (!CDocument::OnNewDocument())
      return FALSE;

   return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CFViewerDoc serialization

void CFViewerDoc::Serialize(CArchive& ar)
{
   if (ar.IsStoring())
      {
      // TODO: add storing code here
      }
   else
      {
      // TODO: add loading code here
      }
}

/////////////////////////////////////////////////////////////////////////////
// CFViewerDoc diagnostics

#ifdef _DEBUG
void CFViewerDoc::AssertValid() const
{
   CDocument::AssertValid();
}

void CFViewerDoc::Dump(CDumpContext& dc) const
{
   CDocument::Dump(dc);
}
#endif //_DEBUG
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
//
// Routines for CFontNode and CFontList follow.
//
// These routines allow us to use a linked list of classes to track all the
// fonts we learned about when we enumerated them.
//
/////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------
//
// StringNode:
//
// Constructor for a node in the type face list.
//
//---------------------------------------------------------------------
CFontNode::CFontNode()
{
   m_string[0] = 0;     // Makes a null string
   m_nextNode = NULL;   // Next node is null since this is a new node.
   return;
}

//---------------------------------------------------------------------
//
// SetString:
//
// Sets the contents of the string portion of a node
//
//---------------------------------------------------------------------
bool CFontNode::SetString(const TCHAR str[LF_FACESIZE])
{
   memcpy( m_string, str, LF_FACESIZE );  // Set the node's string portion
   return TRUE;
}

//---------------------------------------------------------------------
//
// SetNext:
//
// Sets the contents of the next pointer field of a node
//
//---------------------------------------------------------------------
void CFontNode::SetNext(CFontNode *next)
{
   m_nextNode = next;   // Set the next node field.
   return;
}

//---------------------------------------------------------------------
//
// GetNext:
//
// Returns the next node pointer for the current node
//
//---------------------------------------------------------------------
CFontNode * CFontNode::GetNext()
{
   return(m_nextNode);
}

//---------------------------------------------------------------------
//
// GetString:
//
// Returns the string stored in the current node
//
//---------------------------------------------------------------------
TCHAR *CFontNode::GetString()
{
   return(m_string);
}

//---------------------------------------------------------------------
//
// FontList:
//
// Constructor for a potential list of StringNodes
//
//---------------------------------------------------------------------
CFontList::CFontList()
{
   m_start = NULL;   // A new list has no start or end yet, nor any nodes
   m_last = NULL;
   m_nodeCount = 0;
   return;
}

//---------------------------------------------------------------------
//
// NewString:
//
// Creates a new node and adds it to the list, sorted in ascending fashion
// by string name.
//
//---------------------------------------------------------------------
bool CFontList::NewFont(const TCHAR string[LF_FACESIZE], 
               int faceFlag, int fontType, LOGFONT logFontData)
{
   CFontNode *ptr = new CFontNode;  // Create the node
   CFontNode *curPtr, *prevPtr;     // Set up some pointers
   int result;                      // Needed for comparison
   
   ptr->SetString(string);          // Set string in new node
   ptr->m_faceFlag = faceFlag;      // Set faceFlage and fontType
   ptr->m_fontType = fontType;      // (see CFViewerView.c, EnumerateMyFonts())
   ptr->m_selected = 0;             // This font is not currently selected
   ptr->m_logFont = logFontData;    // Copy of the font structure
   ptr->m_selected = 0;             // Indicate that it's not selected

   if (m_start == NULL)       // If this is first node in list, set it up
      {
      m_start = ptr;
      m_last = ptr;
      }
   else                       // Only sort if not first node to add to list
      {
      // 
      // Let's sort the type faces by name
      // This requires starting at the front of the list,
      // and comparing the passed in string with the
      // one in the current node. If the new string is smaller,
      // we insert it before the node we're comparing.
      //
      curPtr = prevPtr = m_start;
      while (curPtr != NULL)
         {
         result = strcmp(string, curPtr->GetString());
         if (result < 0)
            {
            //
            // Insert node before one we're comparing
            //
            if (curPtr == m_start)     // If on first node
               {
               ptr->SetNext(m_start);  // insert as new first
               m_start = ptr;
               }
            else
               {
               prevPtr->SetNext(ptr);  // Point previous to new
               ptr->SetNext(curPtr);   // Point new to current
               }
            break;                     // We're done
            }
         prevPtr = curPtr;             // Move up prev ptr to current
         curPtr = curPtr->GetNext();   // Go to next in list
         }

      if (curPtr == NULL)        // If at end of list...
         {
         prevPtr->SetNext(ptr);  // Add new node there
         m_last = ptr;           // Flag the last
         }
      }

   m_nodeCount++;                // Increment the node count

   return TRUE;
}

//---------------------------------------------------------------------
//
// GetStart:
//
// Returns pointer to start of node list
//
//---------------------------------------------------------------------
CFontNode *CFontList::GetStart()
{
   return(m_start);
}

//---------------------------------------------------------------------
//
// GetNumNodes:
//
// Returns the total number of nodes in the list
//
//---------------------------------------------------------------------
int CFontList::GetNumNodes()
{
   return(m_nodeCount);
}
