//---------------------------------------------------------------------
//
// FViewerDoc.h : interface of the CFViewerDoc class
//
// Part of the FontViewer Utility, which first appeared
// in the May 26, 1998 issue of PC Magazine, U.S. Edition.
// Copyright (C) 1998 by Ziff-Davis, Inc. 
// Programmed by Jake Richter.
//
// The programs presented in PC Magazine can be copied but is copyright.
// You may make copies for others if no charge is involved, but copies
// or using any portion for any commerical purpose is strictly prohibited.
//
// For support, go to http://www.pcmag.com/discuss.htm and select 
// the Utilities area.
//
//---------------------------------------------------------------------

#if !defined(AFX_FONTVIEWDOC_H__F0A23B6D_8BF4_11D1_B24F_0080ADB29FD7__INCLUDED_)
#define AFX_FONTVIEWDOC_H__F0A23B6D_8BF4_11D1_B24F_0080ADB29FD7__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


//---------------------------------------------------------------------
//
// StringNode Class - defines a node in our string list
//
//---------------------------------------------------------------------
class CFontNode
{
public:
   CFontNode();      // Constructor
   bool     SetString(const TCHAR str[LF_FACESIZE]);
   void     SetNext(CFontNode *next);
   CFontNode*  GetNext();
   TCHAR*      GetString();

public: // Exposed data members in a node
   int      m_faceFlag;    // Uses our bitmask to flag native font types
   int      m_fontType;    // As returned by EnumFontFam
   int      m_selected;    // Indicates a font that's selected
   LOGFONT  m_logFont;     // The font's LOGFONT structure

private:
   CFontNode*  m_nextNode; // Pointer to next node
   TCHAR    m_string[LF_FACESIZE];  // Official font name
};

//---------------------------------------------------------------------
//
// CFontList Class - defines our string list
//
//---------------------------------------------------------------------
class CFontList
{
public:
   CFontList();
   bool NewFont(const TCHAR string[LF_FACESIZE], int faceFlag, int fontType, LOGFONT logFontData);
   CFontNode *GetStart();
   int GetNumNodes();
private:
   CFontNode *m_start;     // Start of font list
   CFontNode *m_last;      // End of font list
   int m_nodeCount;        // Number of nodes in list
};


//---------------------------------------------------------------------
//
// CFViewerDoc - derived from CDocument
//
//---------------------------------------------------------------------
class CFViewerDoc : public CDocument
{
protected: // create from serialization only
   CFViewerDoc();
   DECLARE_DYNCREATE(CFViewerDoc)

// Attributes
public:
   CFontList faceList;     // List of type faces

// Operations
public:

// Overrides
   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(CFViewerDoc)
   public:
   virtual BOOL OnNewDocument();
   virtual void Serialize(CArchive& ar);
   //}}AFX_VIRTUAL

// Implementation
public:
   virtual ~CFViewerDoc();
#ifdef _DEBUG
   virtual void AssertValid() const;
   virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
   //{{AFX_MSG(CFViewerDoc)
      // NOTE - the ClassWizard will add and remove member functions here.
      //    DO NOT EDIT what you see in these blocks of generated code !
   //}}AFX_MSG
   DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_FONTVIEWDOC_H__F0A23B6D_8BF4_11D1_B24F_0080ADB29FD7__INCLUDED_)
