//---------------------------------------------------------------------
//
// FViewerView.h : interface of the CFViewerView class
//
// Part of the FontViewer Utility, which first appeared
// in the May 26, 1998 issue of PC Magazine, U.S. Edition.
// Copyright (C) 1998 by Ziff-Davis, Inc. 
// Programmed by Jake Richter.
//
// The programs presented in PC Magazine can be copied but is copyright.
// You may make copies for others if no charge is involved, but copies
// or using any portion for any commerical purpose is strictly prohibited.
//
// For support, go to http://www.pcmag.com/discuss.htm and select 
// the Utilities area.
//
//---------------------------------------------------------------------
#include "FVSetDlg.h"
#include "DispFntDlg.h"

#if !defined(AFX_FONTVIEWVIEW_H__F0A23B6F_8BF4_11D1_B24F_0080ADB29FD7__INCLUDED_)
#define AFX_FONTVIEWVIEW_H__F0A23B6F_8BF4_11D1_B24F_0080ADB29FD7__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//---------------------------------------------------------------------
//
// DisplayedFont Class - defines our object for determining what's being
// display where on the display
//
//---------------------------------------------------------------------
class CDisplayedFont
{
public:
   CDisplayedFont();
   int         m_startY;      // Starting Y position of font line
   int         m_endY;        // Ending Y position
   int         m_styleFlag;   // Style flag (for rendering variations
   BOOL        m_selected;    // Is it selected?
   CString     m_fullName;    // Full font name, including variations
   CFontNode*  m_fontNode;    // FontNode pointer for font.

private:
};

//---------------------------------------------------------------------
//
// FViewerView - Our CView derivative
//
//---------------------------------------------------------------------
class CFViewerView : public CView
{
protected: // create from serialization only
   CFViewerView();
   DECLARE_DYNCREATE(CFViewerView)

// Attributes
public:
   CFViewerDoc* GetDocument();

// Values
public:
   int      maxScroll;        // The maximum scroll position. Set in OnDraw()
   int      maxFonts;         // The maximum # of fonts. Set in FilterFonts()
   int      scrollPos;        // Currently display scrolling position
   int      scrollPageSize;   // Number of lines of text in a scroll
   int      pointSizeTenths;  // Start with a 12 point font
   CString  testString;       // Our initial text string
   UINT     m_fontFilter;     // Our bit flag filter
   CArray<CDisplayedFont, CDisplayedFont> m_DispArray; // Our displayed font table
   int      m_curMinY;        // Our highlighted font's current Minimum Y location
   int      m_curMaxY;        // Our highlighted font's current Maximum Y location
   CFontNode* m_curFontNode;  // Our highlighted font's information
   int      m_curStyleFlag;   // Our highlighted font's style flag
   CString  m_curFullName;    // Our highlighted font's full name
   BOOL     m_curSelected;    // Our highlighted font's selection state
   int      m_curDispIndex;   // Index into displayed font table.
   int		m_curDispPos;	  // Desired display position of highlight after key moves
   BOOL     m_fontHilit;      // Flag indicating a font is highlighted
   int      m_renderVariations;  // The number of valid rendering styles per font
   int      m_rendVarTypes[4];// The valid rendered font styles
   int      m_rendCompound;   // Compound valid rendered font styles
   int      m_splitX;         // The X value of the splitter bar
   BOOL     m_splitTracking;  // If TRUE, we're tracking a split drag.
   BOOL     m_splitCursor;    // If TRUE, we've set the split cursor
   int      m_splitTrackLastX;// Previous X tracking value
   HCURSOR  m_hCursorSizeWE;  // Our splitter cursor
   CFontNode* m_lastSelected; // Point to last selected font in fontNode list
   int      m_lastSelectedStyle; // Style of last selected font
   BOOL     m_viewSelectMode; // Are we in view selected mode?
   int      m_printLineNum;   // The line of text we're printing.
   int      m_printPageNum;   // The page we're printing on
   int      m_totalFonts;     // Total number of fonts available

// Operations
public:

// Overrides
   // ClassWizard generated virtual function overrides
   //{{AFX_VIRTUAL(CFViewerView)
	public:
   virtual void OnDraw(CDC* pDC);  // overridden to draw this view
   virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
   virtual void OnInitialUpdate();
   virtual void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo = NULL);
   virtual void PostNcDestroy();
	protected:
   virtual void OnPrint(CDC* pDC, CPrintInfo* pInfo);
   virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
   virtual ~CFViewerView();
#ifdef _DEBUG
   virtual void AssertValid() const;
   virtual void Dump(CDumpContext& dc) const;
#endif
   CString  CharSetString(int charSet);
protected:
   CFVSetDlg   m_SetDlg;      // Our dialog has its own class (FVSetDlg.*)
   CDispFntDlg m_DispFntDlg;  // Same goes for the DispFontDlg

// Implementation
   void  FilterFonts();
   void  DrawBox(CRect rect, BOOL erase, CDC *pDC);
   void  DrawVSplitter(int x, CDC *pDC);
   void  DrawSelect(CRect rect, CDC *pDC);
   void  ErrBadStyleVariation();
   void  ErrBadFontType();
   void  ErrBadCharacterSet();
   void  ErrBadAttributeFilter();
   void  ClearSelected();
   void  SelectAscending(CFontNode* startNode, int startStyle,
                     CFontNode* endNode, int endStyle);
   void  ProcessDispFontPos(CPoint point, CDC *pDC);


// Generated message map functions
protected:
   //{{AFX_MSG(CFViewerView)
   afx_msg void OnButtonBitmap();
   afx_msg void OnUpdateButtonBitmap(CCmdUI* pCmdUI);
   afx_msg void OnButtonBold();
   afx_msg void OnUpdateButtonBold(CCmdUI* pCmdUI);
   afx_msg void OnButtonNonANSI();
   afx_msg void OnUpdateButtonNonANSI(CCmdUI* pCmdUI);
   afx_msg void OnButtonItalic();
   afx_msg void OnUpdateButtonItalic(CCmdUI* pCmdUI);
   afx_msg void OnButtonPlain();
   afx_msg void OnUpdateButtonPlain(CCmdUI* pCmdUI);
   afx_msg void OnButtonTruetype();
   afx_msg void OnUpdateButtonTruetype(CCmdUI* pCmdUI);
   afx_msg void OnKybdDown();
   afx_msg void OnKybdEnd();
   afx_msg void OnKybdHome();
   afx_msg void OnKybdPgdn();
   afx_msg void OnKybdPgup();
   afx_msg void OnKybdUp();
   afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
   afx_msg void DoFVSetDlg();
   afx_msg void OnToolStringBoxList();
   afx_msg void OnToolStringBoxEdit();
   afx_msg void OnToolStringBoxDropDown();
   afx_msg void OnToolStringBoxSetFocus();
   afx_msg void OnToolPtSizeBoxList();
   afx_msg void OnMouseMove(UINT nFlags, CPoint point);
   afx_msg void OnButtonModern();
   afx_msg void OnUpdateButtonModern(CCmdUI* pCmdUI);
   afx_msg void OnButtonOther();
   afx_msg void OnUpdateButtonOther(CCmdUI* pCmdUI);
   afx_msg void OnButtonSansserif();
   afx_msg void OnUpdateButtonSansserif(CCmdUI* pCmdUI);
   afx_msg void OnUpdateButtonScript(CCmdUI* pCmdUI);
   afx_msg void OnButtonScript();
   afx_msg void OnButtonSerif();
   afx_msg void OnUpdateButtonSerif(CCmdUI* pCmdUI);
   afx_msg void OnButtonViewselect();
   afx_msg void OnUpdateButtonViewselect(CCmdUI* pCmdUI);
   afx_msg void OnButtonAnsi();
   afx_msg void OnUpdateButtonAnsi(CCmdUI* pCmdUI);
   afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
   afx_msg void OnShowCharSet();
   afx_msg void OnShowAlphaNum();
   afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
   afx_msg void OnViewSelected();
   afx_msg void OnButtonBoldItal();
   afx_msg void OnUpdateButtonBoldItal(CCmdUI* pCmdUI);
   afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
   afx_msg void OnKillFocus(CWnd* pNewWnd);
   afx_msg void OnUpdateViewSelected(CCmdUI* pCmdUI);
   afx_msg void OnKybdCtrlSpace();
   afx_msg void OnKybdShiftSpace();
   afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	//}}AFX_MSG
   DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in FViewerView.cpp
inline CFViewerDoc* CFViewerView::GetDocument()
   { return (CFViewerDoc*)m_pDocument; }
#endif

//-------------
//
// Defines for Font Flag
//
//-------------
#define     FNT_TRUETYPE   0x0001
#define     FNT_BITMAP     0x0002
#define     FNT_ANSI    0x0004
#define     FNT_NONANSI    0x0008
#define     FNT_PLAIN      0x0010
#define     FNT_BOLD    0x0020
#define     FNT_ITALIC     0x0040
#define     FNT_BOLDITAL   0x0080      // Only used in the rendVarTypes array
#define     FNT_SANSSERIF  0x0100
#define     FNT_SERIF      0x0200
#define     FNT_MODERN     0x0400
#define     FNT_SCRIPT     0x0800
#define     FNT_OTHER      0x1000
#define     FNT_DISPLAY    0x8000
#define     FNT_MSKFACES   (FNT_PLAIN | FNT_BOLD | FNT_ITALIC | FNT_BOLDITAL)
#define     FNT_MSKSETS    (FNT_ANSI | FNT_NONANSI)
#define     FNT_MSKCLASS   (FNT_TRUETYPE | FNT_BITMAP)
#define     FNT_MSKFAMS    (FNT_SANSSERIF | FNT_SERIF | FNT_MODERN | FNT_SCRIPT | FNT_OTHER)

#define     LOGX_PTS(x)    (((x) * xPerInch) / 72)    // Used in OnPrint for spacing
#define     LOGY_PTS(y)    (((y) * yPerInch) / 72)

#define		DISP_NONE    	0		// Used after a point size change
#define		DISP_TOP  		1		// an invalidate when the arrow keys are used to move hlite
#define		DISP_BOT  		2
#define		DISP_SAME    	3		// Used to restore the current highlight position after

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_FONTVIEWVIEW_H__F0A23B6F_8BF4_11D1_B24F_0080ADB29FD7__INCLUDED_)

/////////////////////////////////////////////////////////////////////////////
